
import speed from "./demo-data-speed.json";
import steering from "./demo-data-steering.json";
import throttle from "./demo-data-throttle.json";

// Import LightningChartJS
const lcjs = require('@arction/lcjs')

// Import xydata
const xydata = require('@arction/xydata')

// Extract required parts from LightningChartJS.
const {
    lightningChart,
    UIElementBuilders,
    UIOrigins,
    PalettedFill,
    LUT,
    ColorRGBA,
    emptyLine,
    AxisTickStrategies,
    Themes,
    synchronizeAxisIntervals,
    AxisScrollStrategies,
    LegendBoxBuilders,
    SolidFill,
    SolidLine
} = lcjs

const { createProgressiveFunctionGenerator,createProgressiveTraceGenerator } = xydata

let license = undefined
try {
    license = '0002-aAgQt777ulf8GBrj/+3CiDwJJ85sBwAJELGq47hD-MEUCIDpBIa0oWQ6OVJJjLAlFlFj0gj41uUyLHJS1jZzngeJIAiEAuBX15C5WnqrehXGhCFVVADA8v6okC3Ff54m9ogK5r0Y='
} catch (e) {}

// Create dashboard which will host all chart and UI elements
const db = lightningChart({
    license: license,
}).Dashboard({
    theme: Themes.darkGold,
    numberOfRows: 3,
    numberOfColumns: 2,
})


//***********************CREATE RACE CHART***********************
const raceChart = db
    .createChartXY({
        theme: Themes.darkGold,
        columnIndex: 0,
        rowIndex: 0,
        columnSpan: 1,
        rowSpan: 2,
    })
    .setTitle('Loading example data ...')
    .setAutoCursor((cursor) => cursor.setTickMarkerXVisible(false).setTickMarkerYVisible(false))

 const axisXC1 = raceChart.getDefaultAxisX();
 const axisYC1 = raceChart.getDefaultAxisY();
 raceChart
   .getDefaultAxes()
   .forEach((axis) =>
     axis.setTickStrategy(AxisTickStrategies.Empty).setStrokeStyle(emptyLine)
   );
 
 const lineSeries = raceChart
   .addLineSeries({ individualLookupValuesEnabled: true })
   .setCursorSolveBasis("nearest")
   .setCursorInterpolationEnabled(false)
   .setStrokeStyle((stroke) => stroke.setThickness(10));

 const dataSources = [
   {
     name: "Speed",
     file: speed,
     lut: new LUT({
       steps: [
         { value: 0, color: ColorRGBA(255, 255, 255) },
         { value: 25, color: ColorRGBA(255, 215, 0) },
         { value: 50, color: ColorRGBA(255, 0, 0) },
       ],
       interpolate: true,
       units: "km/h",
     }),
     format: (value) => `${value.toFixed(1)} km/h`,
   },
   {
     name: "Steering",
     file: steering,
     lut: new LUT({
       steps: [
         { value: -100, color: ColorRGBA(255, 255, 0) },
         { value: 0, color: ColorRGBA(255, 255, 255) },
         { value: 100, color: ColorRGBA(0, 0, 255) },
       ],
       interpolate: true,
       units: "%",
     }),
     format: (value) => `${value.toFixed(1)} %`,
   },
   {
     name: "Throttle",
     file: throttle,
     lut: new LUT({
       steps: [
         { value: 0, color: ColorRGBA(255, 255, 255) },
         { value: 50, color: ColorRGBA(255, 215, 0) },
         { value: 100, color: ColorRGBA(255, 0, 0) },
       ],
       interpolate: true,
       units: "%",
     }),
     format: (value) => `${value.toFixed(1)} %`,
   },
 ];
 
 let legend;
 const displayDataSource = async (sourceName) => {
   // CLeanup previously displayed data source
   if (legend) {
     legend.dispose();
     legend = undefined;
   }
 
   const dataSource = dataSources.find((item) => item.name === sourceName);
   let data = dataSource.file;
 
   lineSeries
     .clear()
     .setName(sourceName)
     .add(data)
     .setStrokeStyle((stroke) =>
       stroke.setFillStyle(new PalettedFill({ lut: dataSource.lut }))
     )
     .setCursorResultTableFormatter((builder, _, __, ___, dataPoint) =>
       builder
         .addRow(sourceName)
         .addRow(dataPoint.value && dataSource.format(dataPoint.value))
     );
 
     raceChart.setTitle(`${sourceName} progression`);
 
   legend = raceChart.addLegendBox().add(raceChart);
 };

 displayDataSource("Speed");
    

//***********************CREATE REAL TIME MONITORING***********************

const DATA_FREQUENCY_HZ = 1000
const CHANNELS_AMOUNT = 10
const xIntervalMax = 30 * DATA_FREQUENCY_HZ
const channelIntervalY = 2 // [-1, 1]

const monitorChart = db
.createChartXY({
    columnIndex: 0,
    rowIndex: 2,
    columnSpan: 2,
})
.setPadding({ right: 40 })
.setTitle(`Multi-channel real-time monitoring (${CHANNELS_AMOUNT} chs, ${DATA_FREQUENCY_HZ} Hz)`)

const axisXC2 = monitorChart
.getDefaultAxisX()
.setScrollStrategy(AxisScrollStrategies.progressive)
.setInterval({ start: -xIntervalMax, end: 0, stopAxisAfter: false })
.setTitle('Data points per channel')
    
const axisYC2 = monitorChart
    .getDefaultAxisY()
    .setTickStrategy(AxisTickStrategies.Empty)
    .setTitle('< Channels >')
    .setScrollStrategy(AxisScrollStrategies.expansion)
    .setInterval({ start: -channelIntervalY / 2, end: CHANNELS_AMOUNT * channelIntervalY, stopAxisAfter: false })

const series = new Array(CHANNELS_AMOUNT).fill(0).map((_, iChannel) => {
    // Create line series optimized for regular progressive X data.
    const nSeries = monitorChart
        .addLineSeries({
            dataPattern: {
                // pattern: 'ProgressiveX' => Each consecutive data point has increased X coordinate.
                pattern: 'ProgressiveX',
                // regularProgressiveStep: true => The X step between each consecutive data point is regular (for example, always `1.0`).
                regularProgressiveStep: true,
            },
        })
        .setName(`Channel ${iChannel + 1}`)
        // Use -1 thickness for best performance, especially on low end devices like mobile / laptops.
        .setStrokeStyle((style) => style.setThickness(-1))
        .setMouseInteractions(false)
        // Enable automatic data cleaning.
        .setDataCleaning({ minDataPointCount: xIntervalMax })

    // Add custom tick for each channel.
    monitorChart
        .getDefaultAxisY()
        .addCustomTick()
        .setValue((CHANNELS_AMOUNT - (1 + iChannel)) * channelIntervalY)
        .setTextFormatter(() => `Channel ${iChannel + 1}`)
        .setGridStrokeStyle(
            new SolidLine({
                thickness: 1,
                fillStyle: new SolidFill({ color: ColorRGBA(255, 255, 255, 60) }),
            }),
        )

    return nSeries
})

// Add LegendBox.
monitorChart
    .addLegendBox()
    .add(monitorChart)
    // Dispose example UI elements automatically if they take too much space. This is to avoid bad UI on mobile / etc. devices.
    .setAutoDispose({
        type: 'max-width',
        maxWidth: 0.3,
    })

// Define unique signals that will be used for channels.
const signals = [
    { length: 100 * 2 * Math.PI, func: (x) => Math.sin(x / 100) },
    { length: 100 * 2 * Math.PI, func: (x) => Math.cos(x / 100) },
    {
        length: 200 * 2 * Math.PI,
        func: (x) => Math.cos(x / 200) + Math.sin(x / 100),
    },
    {
        length: 200 * 2 * Math.PI,
        func: (x) => Math.sin(x / 50) + Math.cos(x / 200),
    },
    {
        length: 200 * 2 * Math.PI,
        func: (x) => Math.sin(x / 100) * Math.cos(x / 200),
    },
    { length: 450 * 2 * Math.PI, func: (x) => Math.cos(x / 450) },
    { length: 800 * 2 * Math.PI, func: (x) => Math.sin(x / 800) },
    {
        length: 650 * 2 * Math.PI,
        func: (x) => Math.sin(x / 200) * Math.cos(x / 650),
    },
]

// Generate data sets for each signal.
Promise.all(
    signals.map((signal) =>
        createProgressiveFunctionGenerator()
            .setStart(0)
            .setEnd(signal.length)
            .setStep(1)
            .setSamplingFunction(signal.func)
            .generate()
            .toPromise()
            .then((data) => data.map((xy) => xy.y)),
    ),
).then((dataSets) => {
    // Stream data into series.
    let tStart = window.performance.now()
    let pushedDataCount = 0
    const xStep = 1000 / DATA_FREQUENCY_HZ
    const streamData = () => {
        const tNow = window.performance.now()
        // NOTE: This code is for example purposes (streaming stable data rate without destroying browser when switching tabs etc.)
        // In real use cases, data should be pushed in when it comes.
        const shouldBeDataPointsCount = Math.floor((DATA_FREQUENCY_HZ * (tNow - tStart)) / 1000)
        const newDataPointsCount = Math.min(shouldBeDataPointsCount - pushedDataCount, 1000) // Add max 1000 data points per frame into a series. This prevents massive performance spikes when switching tabs for long times
        const seriesNewDataPoints = []
        for (let iChannel = 0; iChannel < series.length; iChannel++) {
            const dataSet = dataSets[iChannel % dataSets.length]
            const newDataPoints = []
            for (let iDp = 0; iDp < newDataPointsCount; iDp++) {
                const x = (pushedDataCount + iDp) * xStep
                const iData = (pushedDataCount + iDp) % dataSet.length
                const ySignal = dataSet[iData]
                const y = (CHANNELS_AMOUNT - 1 - iChannel) * channelIntervalY + ySignal
                const point = { x, y }
                newDataPoints.push(point)
            }
            seriesNewDataPoints[iChannel] = newDataPoints
        }
        series.forEach((series, iChannel) => series.add(seriesNewDataPoints[iChannel]))
        pushedDataCount += newDataPointsCount
        requestAnimationFrame(streamData)
    }
    streamData()
})

// Measure FPS.
let tStart = window.performance.now()
let frames = 0
let fps = 0
const title = monitorChart.getTitle()
const recordFrame = () => {
    frames++
    const tNow = window.performance.now()
    fps = 1000 / ((tNow - tStart) / frames)
    sub_recordFrame = requestAnimationFrame(recordFrame)

    monitorChart.setTitle(`${title} (FPS: ${fps.toFixed(1)})`)
}
let sub_recordFrame = requestAnimationFrame(recordFrame)
setInterval(() => {
    tStart = window.performance.now()
    frames = 0
}, 5000)

//***********************CREATE MONITORING HEATMAP CHART***********************
const heatmapChart = db
    .createChartXY({
        theme: Themes.darkGold,
        columnIndex: 1,
        rowIndex: 0,
    }).setTitle('Real-Time Chart + DataGrid')

    const exampleTrends = [
        {
            name: 'Trend #1',
        },
        {
            name: 'Trend #2',
        },
        {
            name: 'Trend #3',
        },
    ]
    const exampleTrendsCount = exampleTrends.length
    const exampleDataCount = 10000

const dataGrid = db
    .createDataGrid({ 
        columnIndex: 1,
        rowIndex: 1,
    })
    .setTitle('')
    .setColumnContent(0, ['', ...exampleTrends.map((trend) => trend.name)])
    .setRowContent(0, ['', 'Latest value', 'Previous value', 'value 15 s'])

const seriesXYList = exampleTrends.map((trend) =>
heatmapChart
        .addLineSeries({ dataPattern: { pattern: 'ProgressiveX' } })
        .setDataCleaning({ minDataPointCount: 1 })
        .setName(trend.name),
)
const axisX = heatmapChart
    .getDefaultAxisX()
    .setScrollStrategy(AxisScrollStrategies.progressive)
    .setInterval({ start: -60 * 1000, end: 0, stopAxisAfter: false })
    .setTickStrategy(AxisTickStrategies.Time)

const axisXTop = heatmapChart
    .addAxisX({ opposite: true })
    .setTickStrategy(AxisTickStrategies.Empty)
    .setStrokeStyle(emptyLine)
    .setMouseInteractions(false)
synchronizeAxisIntervals(axisX, axisXTop)
const indicator15s = axisXTop.addCustomTick(UIElementBuilders.AxisTickMajor).setTextFormatter((_) => '-15 s')

const legendC3 = heatmapChart
    .addLegendBox(LegendBoxBuilders.HorizontalLegendBox, { x: heatmapChart.getDefaultAxisX(), y: heatmapChart.getDefaultAxisY() })
    .add(heatmapChart)
const positionLegend = () => {
    legendC3.setOrigin(UIOrigins.CenterBottom).setPosition({
        x: (heatmapChart.getDefaultAxisX().getInterval().start + heatmapChart.getDefaultAxisX().getInterval().end) / 2,
        y: heatmapChart.getDefaultAxisY().getInterval().start,
    })
}
heatmapChart.forEachAxis((axis) => axis.onIntervalChange(positionLegend))

const theme = db.getTheme()
const textFillGood = theme.examples.positiveTextFillStyle
const textFillBad = theme.examples.negativeTextFillStyle
const bgFillGood = theme.examples.positiveBackgroundFillStyle
const bgFillBad = theme.examples.negativeBackgroundFillStyle

Promise.all(
    new Array(exampleTrendsCount).fill(0).map((_) =>
        createProgressiveTraceGenerator()
            .setNumberOfPoints(exampleDataCount)
            .generate()
            .toPromise()
            .then((data) => data.map((xy) => 100 + xy.y)),
    ),
).then((exampleData) => {
    const trendsHistory = exampleTrends.map(() => ({
        previous: 0,
        previous15s: 0,
    }))
    const tStart = Date.now()

    const streamOneSample = (sample, isFirst) => {
        const tNow = Date.now()

        seriesXYList.forEach((series, iTrend) => series.add({ x: tNow - tStart, y: sample[iTrend] }))

        if (isFirst) {
            trendsHistory.forEach((trendHistory, iTrend) => {
                trendHistory.previous15s = sample[iTrend]
            })
            setInterval(() => {
                trendsHistory.forEach((trendHistory, iTrend) => {
                    trendHistory.previous15s = trendHistory.previous
                })
                indicator15s.setValue(Date.now() - tStart)
            }, 1000 * 15)
        }

        sample.forEach((value, iTrend) => {
            const trendHistory = trendsHistory[iTrend]
            const current = value
            const previous = trendHistory.previous
            const previous15s = trendHistory.previous15s
            dataGrid
                // Current
                .setCellContent(1, iTrend + 1, `${current.toFixed(1)}`)
                .setCellTextFillStyle(1, iTrend + 1, current > previous ? textFillGood : textFillBad)
                .setCellBackgroundFillStyle(1, iTrend + 1, current > previous ? bgFillGood : bgFillBad)
                // Previous
                .setCellContent(2, iTrend + 1, `${previous.toFixed(1)}`)
                .setCellTextFillStyle(2, iTrend + 1, previous > previous15s ? textFillGood : textFillBad)
                .setCellBackgroundFillStyle(2, iTrend + 1, previous > previous15s ? bgFillGood : bgFillBad)
                // Previous 15 s
                .setCellContent(3, iTrend + 1, `${previous15s.toFixed(1)}`)

            trendHistory.previous = current
        })   
    }

    
    let iSample = 0
    const sub = setInterval(() => {
        streamOneSample(
            new Array(exampleTrendsCount).fill(0).map((_, iTrend) => exampleData[iTrend][iSample]),
            iSample === 0,
        )

        iSample += 1
        if (iSample >= exampleDataCount) {
            clearInterval(sub)
        }
    }, 50)
})
